
import numpy

from variables import *

class Run (object):
    def __init__(self, name=None, prefix=''):
        if name:
            self.outputf = prefix+'output_'+name+'.dat'
            self.timesf = 'times_'+name+'.py'
            self.configf = 'config_'+name+'.py'
            self.name = name
        else:
            self.outputf = prefix+'output.dat'
            self.timesf = 'times.py'
            self.configf = 'config.py'
            self.name = 'run'
        config = {}
        times = {}
        execfile(self.configf, globals(), config)
        execfile(self.timesf, globals(), times)
        self.times = times['time']
        self.opts = config['opts']
        self.vars = config['used']
        self.i = Config(self, int)
        self.f = Config(self, float)
        
        # detecting number of variables from equation type
        if 'equation' in self.opts:
          self.NVARS = get_NVARS (self.opts['equation'])
        else:
          self.NVARS = 16

    def var(self, v, ts):
        if ts > len(self.times):
            raise Exception, "Timestep out of bounds"
        f = open(self.outputf, 'rb')
        
        n = self.i.NX*self.i.NY*self.i.NZ / (self.i.MULTIX*self.i.MULTIY*self.i.MULTIZ)
        
        f.seek(8*n*(v.i + self.NVARS*ts))
        fd = numpy.fromfile(f, dtype=numpy.float64, count=n)
        f.close()
        fd.shape = (self.i.NX, self.i.NY, self.i.NZ)
        return fd

    def combine(self):
        cshape = (self.i.NX, self.i.NY, self.i.NZ)
        shape = (self.i.NX / self.i.MULTIX, self.i.NY / self.i.MULTIY, self.i.NZ / self.i.MULTIZ)
        
        n = self.i.NX*self.i.NY*self.i.NZ / (self.i.MULTIX*self.i.MULTIY*self.i.MULTIZ)
        cf = open(self.outputf, 'wb')
        p = []
        for i in range(self.i.MULTIX):
            for j in range(self.i.MULTIY):
                for k in range(self.i.MULTIZ):
                    f = open("%s.%d.%d.%d"%(self.outputf, i, j, k), 'rb')
                    p.append((f,i*(self.i.NX / self.i.MULTIX),j*(self.i.NY / self.i.MULTIY),k*(self.i.NZ / self.i.MULTIX)))
        for l in range(self.NVARS*len(self.times)):
            cd = numpy.zeros(cshape, dtype=numpy.float64)
            for f,x,y,z in p:
                d = numpy.fromfile(f, dtype=numpy.float64, count=n)
                d.shape = shape
                cd[x:x+(self.i.NX / self.i.MULTIX), y:y+(self.i.NY / self.i.MULTIY), z:z+(self.i.NZ / self.i.MULTIZ)] = d
            cd.tofile(cf)
        for f,x,y,z in p:
            f.close()
        cf.close()

if __name__ == '__main__':
    
    import sys
    if len(sys.argv) != 2:
        name = None
    else:
        name = sys.argv[1]
    
    r = Run(name)
    if r.vars['MULTIX'] != '1' or r.vars['MULTIY'] != '1' or r.vars['MULTIZ'] != '1':
      r.combine()
      if r.vars['COMPUTE_VARIANCE'] != '0':
        r = Run(name, 'var_')
        r.combine()
