
#include <cstdlib>
#include <iostream>

#include "chaos.h"
#include "rng.h"

static real NORMAL_buffer;
static bool NORMAL_buffer_empty = true;

// generates standard normal random variable using rejection method and Box-Muller transformation
real draw_NORMAL () {
  
  if (!NORMAL_buffer_empty) {
    NORMAL_buffer_empty = true;
    return NORMAL_buffer;
  }
  
  else {
    real U1, U2, r, w, N1, N2;
    
    // rejection method to obtain (U1,U2) - uniform random variable on unit circle
    do {
      // draw two uniform random variable on [-1, 1]
      U1 = 2*draw_UNIFORM() - 1;
      U2 = 2*draw_UNIFORM() - 1;
      // compute radius
      r = sqr(U1) + sqr(U2);
    } while (r > 1.0);
    
    // Box-Muller transformation
    w  = sqrt(-2*log(r)/r);
    N1 = U1 * w;
    N2 = U2 * w;
    
    NORMAL_buffer = N2;
    NORMAL_buffer_empty = false;
    
    return N1;
  }
}

Chaos chaos_draw () {
  Chaos chaos;
  
  // initial data
  for (long_int i=0; i<N_UNIFORM; i++)
    chaos.uniform[i] = draw_UNIFORM();
  for (long_int i=0; i<N_NORMAL; i++)
    chaos.normal[i] = draw_NORMAL();
  
  // topography
  for (long_int i=0; i<T_UNIFORM; i++)
    chaos.t_uniform[i] = draw_UNIFORM();
  for (long_int i=0; i<T_NORMAL; i++)
    chaos.t_normal[i] = draw_NORMAL();
  
  // coefficient k
  for (long_int i=0; i<K_UNIFORM; i++)
    chaos.k_uniform[i] = draw_UNIFORM();
  for (long_int i=0; i<K_NORMAL; i++)
    chaos.k_normal[i] = draw_NORMAL();
  
  return chaos;
}

void chaos_init (long_int offset) {
  // offsetting samples
  for (long_int i=0; i<offset; i++)
    chaos_draw ();
}
