
/* EQUATION SPECIFIC FVM routines
 * Euler (Euler equations of gas dynamics)
 */

///::title:: Euler (Euler equations of gas dynamics)
///::consts:: NO_B=1 GAMMA=(5.0/3.0)

#include "equation.h"

void calc_point_flux(ConservedVars &F, const AllVariables &V, int d)
{
  F.rho = V.U.m[d];
  F.m = V.U.m * V.XU.u[d];
  F.m[d] += V.XU.p;
  F.E = (V.U.E + V.XU.p) * V.XU.u[d];
}

void apply_constraints (ConservedVars &CV, XtraVars &XV)
{
  // Constraints for ConservedVars
  if (CV.rho < 0.0) CV.rho = 0.0;
  if (CV.E   < 0.0) CV.E   = 0.0;
  // Constraints for XtraVars
  if (XV.p   < 0.0) XV.p   = 0.0;
}

real calc_time_step()
{
  vector maxeig;

  maxeig.zero();
  for (int i = 0; i < NXl; i++) {
    for (int j = 0; j < NYl; j++) {
      for (int k = 0; k < NZl; k++) {
        for (int d = 0; d < NDIR; d++) {
          if (!HAVE_DIR(d)) continue;
          maxeig[d] = max( maxeig[d], fabs(Xtra[i][j][k].u[d]) + sqrt( GAMMA * Xtra[i][j][k].p / U[i][j][k].rho ) );
        }
      }
    }
  }
  return CFL/(maxeig[X]/dx[X] + maxeig[Y]/dx[Y] + maxeig[Z]/dx[Z]);
}

void calc_xtra(XtraVars &Xijk, const ConservedVars &Uijk, int i, int j, int k)
{
  real ie = Uijk.E - 0.5*Uijk.m.sqr()/Uijk.rho;
  Xijk.u = Uijk.m / Uijk.rho;
  Xijk.p = (GAMMA-1)*ie;
  if (Xijk.p < 0) {
    std::cerr<<"Negative Pressure p="<<Xijk.p<<"("<<i<<' '<<j<<' '<<k<<") t="<<t<<"\n";
    throw 1;
  }
}

void calc_all_vars(AllVariables &V, const PrimitiveVars &P)
{
  P.toConservedVars(V.U);
  V.XU.p = P.p;
  V.XU.u = P.u;
  if (P.p < 0) {
    std::cerr<<"Negative Pressure\n";
    throw 1;
  }
}

#if INDICATOR == 1
// rho
real indicator(int i, int j, int k, int d)
{
  ConservedVars &Uijk = U[i][j][k];
  return Uijk.rho;
}
#endif
